#!/usr/bin/env python

import datetime
import logging

from django.core.exceptions import ObjectDoesNotExist

from qlbr import LP, TZ
from qlbr.models import Bug, Person, Search, SearchTag, Tag, Task


RUN_START = datetime.datetime.now(TZ)

from sys import argv
if 'reload' in argv:
    RELOAD = True
else:
    RELOAD = False


def get_person(person_link):
    if person_link is None:
        return None
    logging.info('Checking person: %s', person_link)
    try:
        person = Person.objects.get(self_link=person_link)
        logging.info('%s is already stored locally', person_link)
        return person
    except ObjectDoesNotExist:
        logging.info('Retriveing %s from launchpad', person_link)
    lp_person = LP.load(person_link)
    d_person = Person(self_link=lp_person.self_link,
                      web_link=lp_person.web_link,
                      name=lp_person.name,
                      display_name=lp_person.display_name)
    d_person.save()
    return d_person


def get_bug(bug_link, duplicate=False):
    if bug_link is None:
        return None
    logging.info('Checking bug: %s', bug_link)
    try:
        bug = Bug.objects.get(self_link=bug_link)
        if bug.last_update > RUN_START:
            logging.info('%s already gathered this run', bug_link)
            return bug
        else:
            logging.info('Updating %s from launchpad', bug_link)
    except ObjectDoesNotExist:
        logging.info('Retrieving %s from launchpad', bug_link)
    lp_bug = LP.load(bug_link)
    d_owner = get_person(lp_bug.owner_link)
    d_duplicate_of = get_bug(lp_bug.duplicate_of_link, duplicate=True)
    bug_values = {'web_link': lp_bug.web_link,
                  'bug_id': lp_bug.id,
                  'title': lp_bug.title,
                  'date_last_updated': lp_bug.date_last_updated,
                  'date_created': lp_bug.date_created,
                  'owner': d_owner,
                  'duplicate_of': d_duplicate_of
                  }
    d_bug, _created = Bug.objects.update_or_create(self_link=lp_bug.self_link,
                                                   defaults=bug_values)
    d_bug.save()
    for tag in lp_bug.tags:
        d_tag, _created = Tag.objects.get_or_create(tag=tag)
        d_bug.tags.add(d_tag)
    d_bug.save()
    if duplicate:
        get_task(lp_bug.bug_tasks[0], ['duplicate'])
    return d_bug


def get_task(task, search_tags=[]):
    if isinstance(task, basestring):
        task_link = task
    else:
        task_link = task.self_link
    logging.info('Checking task: %s', task_link)
    try:
        d_task = Task.objects.get(self_link=task_link)
        if d_task.last_update > RUN_START:
            logging.info('%s already gathered this run', task_link)
            for tag in search_tags:
                d_tag, _created = SearchTag.objects.get_or_create(
                    search_tag=tag)
                d_task.search_tags.add(d_tag)
            d_task.save()
            return d_task
        else:
            logging.info('Updating %s from launchpad', task_link)
    except ObjectDoesNotExist:
        logging.info('Retrieving %s from launchpad', task_link)
    if isinstance(task, basestring):
        task = LP.load(task)
    d_bug = get_bug(task.bug_link)
    d_owner = get_person(task.owner_link)
    d_assignee = get_person(task.assignee_link)
    task_values = {'web_link': task.web_link,
                   'importance': task.importance,
                   'status': task.status,
                   'date_created': task.date_created,
                   'bug_target_name': task.bug_target_name,
                   'bug_target_display_name': task.bug_target_display_name,
                   'bug': d_bug,
                   'owner': d_owner,
                   'assignee': d_assignee
                   }
    d_task, _created = Task.objects.update_or_create(self_link=task.self_link,
                                                     defaults=task_values)
    d_task.save()
    for tag in search_tags:
        d_tag, _created = SearchTag.objects.get_or_create(search_tag=tag)
        d_task.search_tags.add(d_tag)
    d_task.save()
    return d_task


def search_tasks(defaults={
        'omit_duplicates': False,
        'status': ['New',
                   'Confirmed',
                   'Triaged',
                   'In Progress',
                   'Fix Committed',
                   'Opinion',
                   'Invalid',
                   "Won't Fix",
                   'Fix Released',
                   'Expired',
                   'Incomplete',
                   'Incomplete (with response)',
                   'Incomplete (without response)'
                   ]}):
    for search in Search.objects.all():
        logging.info('Running search: %s', search)
        criteria = defaults
        if not RELOAD:
            criteria.update(modified_since=search.last_update.isoformat())
        criteria.update(search.criteria_dict)
        for project in search.projects.values_list('project', flat=True):
            logging.info('Searching project: %s', project)
            lp_project = LP.projects[project]
            for task in lp_project.searchTasks(**criteria):
                get_task(task, search.search_tags.values_list(
                    'search_tag', flat=True))
        search.last_update = RUN_START
        search.save()

search_tasks()
