import re

from django.db import models


class SearchTag(models.Model):
    search_tag = models.CharField(max_length=255, primary_key=True)

    def __str__(self):
        return self.search_tag


class Item(models.Model):
    item = models.CharField(max_length=255, primary_key=True)

    def __str__(self):
        return self.item


class Criterion(models.Model):
    key = models.CharField(max_length=255)
    is_list = models.BooleanField()
    value = models.CharField(max_length=255)
    values = models.ManyToManyField('Item')

    def __str__(self):
        return self.key

    @property
    def get(self):
        if self.is_list:
            return list(self.values.values_list('item', flat=True))
        else:
            return self.value


class Project(models.Model):
    project = models.CharField(max_length=255, primary_key=True)

    def __str__(self):
        return self.project


class Search(models.Model):
    name = models.CharField(max_length=255)
    search_tags = models.ManyToManyField('SearchTag')
    projects = models.ManyToManyField('Project')
    criteria = models.ManyToManyField('Criterion')
    last_update = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name

    @property
    def criteria_dict(self):
        criteria = {}
        for criterion in self.criteria.iterator():
            criteria[criterion.key] = criterion.get
        return criteria


class Person(models.Model):
    self_link = models.CharField(max_length=255, primary_key=True)
    web_link = models.CharField(max_length=255)
    name = models.CharField(max_length=255)
    display_name = models.CharField(max_length=255)
    last_update = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.display_name


class Tag(models.Model):
    tag = models.CharField(max_length=255, primary_key=True)
    matcher = re.compile('rls-[a-z]-incoming')

    def __str__(self):
        return self.tag

    @property
    def release(self):
        if self.matcher.search(self.tag):
            return True
        else:
            return False


class Bug(models.Model):
    self_link = models.CharField(max_length=255, primary_key=True)
    web_link = models.CharField(max_length=255)
    bug_id = models.IntegerField()
    tags = models.ManyToManyField('Tag')
    date_last_updated = models.DateTimeField()
    date_created = models.DateTimeField()
    owner = models.ForeignKey('Person', on_delete=models.PROTECT)
    duplicate_of = models.ForeignKey(
        'self', blank=True, null=True, on_delete=models.PROTECT)
    title = models.CharField(max_length=255)
    last_update = models.DateTimeField(auto_now=True)

    def __str__(self):
        return str(self.bug_id)

    @property
    def tag_list(self):
        return ' '.join(self.tags.values_list('tag', flat=True))

    @property
    def triaged(self):
        if self.task_set.filter(status='Fix Released').exists():
            return True
        if any([tag.release for tag in self.tags.all()]):
            return True
        return False


class Task(models.Model):
    self_link = models.CharField(max_length=255, primary_key=True)
    web_link = models.CharField(max_length=255)
    importance = models.CharField(max_length=255)
    status = models.CharField(max_length=255)
    date_created = models.DateTimeField()
    bug_target_name = models.CharField(max_length=255)
    bug_target_display_name = models.CharField(max_length=255)
    bug = models.ForeignKey('Bug', on_delete=models.PROTECT)
    owner = models.ForeignKey('Person', on_delete=models.PROTECT,
                              related_name='task_owner')
    assignee = models.ForeignKey(
        'Person', blank=True, null=True, on_delete=models.PROTECT,
        related_name='task_assignee')
    search_tags = models.ManyToManyField('SearchTag')
    last_update = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.web_link

    @property
    def triaged(self):
        if self.bug.triaged:
            return True
        if self.status == 'New':
            return False
        return True
